#ifndef __IUF_GAMESESSION
#define __IUF_GAMESESSION

#include <vcg/Point3.h>
#include <list>
#include "GameObj.h"
#include "Bullet.h"
#include "Ast.h"
#include "Ship.h"

/** This class represent a Game session.
It contain all the data needed during the game play, 
active game objects, score, lives and so on.
Version History {\small 
\begin{itemize}
\item 0.1 First Release.
\item 0.1s Modified Shoot to manage multiple shots .
\item 0.3 Added collision detection between ship and asteroids,
 \Ref(AstShipCollision), and between bullets and Asteroids,
 \Ref{BulletAstCollision}. Modified \Ref{Update} to manage the 
 effect of the various collisions (death of the player, explosion 
 and splitting of the asteroid).
\item 0.3s Added PauseFlag and implemented Pause()
\end{itemize}
}


*/
using namespace vcg;
using namespace std;
class GameSession
{
 public:

	 GameSession()
	 {
		 PauseFlag=false;
		 SpaceSize=100;
	 }

 /** The list of all the active bullets
 @type list<Bullet *> */
	 list<Bullet *> ActiveBullet;
 
 /** The list of all the active Asteroid 
 @type list<Ast*> */
	 list<Ast *> ActiveAst;
 
 /** The list of all the generic active objects 
 @type list<GameObj *> */
	 list<GameObj *> ActiveObj;
 
  /// The only Spaceship in game.
  Ship *S;
 
	/** This function updates All the active objects.
	This is done by traversig the three list (ActiveAst, 
	ActiveBullet and ActiveObj) and calling the Update virtual 
	function for each of them. If an object become inactive 
	after an update it is removed by the list and deallocated here.
	During the traversal of ActiveAst list we also check for 
	\Ref{AstShipCollision}.
	During the traversal of ActiveBullet list we check 
	for \Ref{BulletAstCollision} Note that this function traverse 
	again the ActiveAst list (quadratically!). 
*/
 	void Update(int dt);
 
  /** This function draws all the active objects.
	The ActiveObj list (the one containing explosions) 
	is drawn as the last list to correctly draw the explosion 
	transparency effects.
	*/
  void Draw();
 
  /// This function clip the position of an object against the screen boundary
  void Clip(Point3f &p)
  {
	 if(p.x() > SpaceSize) p.x() -=2*SpaceSize; 
   if(p.y() > SpaceSize) p.y() -=2*SpaceSize; 
   if(p.x() <-SpaceSize) p.x() +=2*SpaceSize; 
   if(p.y() <-SpaceSize) p.y() +=2*SpaceSize; 
 };
 
  /** Shoot a bullet.
 What kind of (and how many!) bullet is shot depends on the current type of 
 ship stored in member S;
 */
 void Shoot();

 
 /// Start the game
 void Start();
 
 /// Pause the game
 void Pause ()
 {PauseFlag =! PauseFlag;}

 /// Game Over;
 void End();

 /// Screen Width
 int ScreenW;
 /// Screen Height
 int ScreenH;
 ///The size of the Game Space (in mt)
 int SpaceSize;

 /** Check the collision between the asteroids and the ship
 If the ship is hit by an asteroid the ship is inactivated 
 (e.g. it will not be drawn) for a couple of seconds by setting 
 the deadpause member variable.
	*/
 
 void AstShipCollision(Ast *a);

 /** Check the collision between a bullet and all the asteroids.
 If an asteroid is hit the bullet and the asteroid are deactivated,
 a new explosion is added to the ActiveObj list and two new smaller
 asteroids are spawn.
 */
 void BulletAstCollision(Bullet *b);

	/** Used to delay the creation of the new ship. 
	After a ship-asteroid collision the AstShipCollision function set this to wait for two secs.
	*/
	int deadpause;	
/// True if the game is paused;
	bool PauseFlag;
};

#endif // ifndef __IUF_GAMESESSION